﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;

namespace LinqTest
{
    class Program
    {
        static void Main(string[] args)
        {
          //Object initializer, with anonymous type:
            var cars = new[]
            {
                new {RegNo = "XX12345", Model = "BMW", Year = 2009},
                new {RegNo = "YY12345", Model = "Audi", Year = 2010},
                new {RegNo = "ZZ12345", Model = "VW", Year = 2013},
                new {RegNo = "TT12345", Model = "Mercedes", Year = 2014}
            };

          //Object initializer, with class Car:
            var cars1 = new Car[]
            {
                new Car {RegNo = "XX12345", Model = "BMW", Year = 2009},
                new Car {RegNo = "YY12345", Model = "Audi", Year = 2010},
                new Car {RegNo = "ZZ12345", Model = "VW", Year = 2013},
                new Car {RegNo = "TT12345", Model = "Mercedes", Year = 2014}

            };

            
            // The oldschool method without lamda-expressions and Linq 
            var carOldMethod = new List<string>();
            foreach (var car in cars)
            {
                carOldMethod.Add(car.Model);
            }
            foreach (var c in carOldMethod)
            {
                Console.WriteLine(c);
            }

            Console.WriteLine();

            //The raw method - to show whats going on behind the sceens
            //Select is an extension method that returns an enumerable object, 
            //that will fetch the data identifyed by the Select method
            IEnumerable<string> carModel = cars.Select(car => car.Model);
            IEnumerator<string> enumerator = carModel.GetEnumerator();
            while (enumerator.MoveNext())
            {
                Console.WriteLine(enumerator.Current);
            }


            //Here we hide the IEnumerable by using implicit typed variable and foreach
            //Foreach can only be used on collections that implements IEnumerable, like List<>, String, etc.
            //carModel2 could simply be replaced with carModel1
            var carModel2 = cars1.Select(car => car.Model);
            foreach (var c in carModel2)
            {
                Console.WriteLine(c);
            }

            Console.WriteLine();

            
            //Here we are using query operators - looks more like a sql-query (easier to read)
            var carModelQuery = from car in cars select car.Model;
            foreach (var c in carModelQuery)
            {
                Console.WriteLine(c);
            }

            Console.WriteLine();

            //Old school
            var carOldMethod1 = new List<string>();
            foreach (var car in cars)
            {
                if (car.Year>2010)
                carOldMethod1.Add(car.Model);
            }
            foreach (var c in carOldMethod1)
            {
                Console.WriteLine(c);
            }

            Console.WriteLine();

            //With Lamda-expression
            var carModel1 = cars.Where(car=>car.Year>2010).Select(car => car.Model);
            foreach (var c in carModel1)
            {
                Console.WriteLine(c);
            }

            Console.WriteLine();
          
            //with Query operators
            var carModelQuery1 = from car in cars
                                 where car.Year>2010
                                 select car.Model;
            foreach (var c in carModelQuery1)
            {
                Console.WriteLine(c);
            }

            Console.WriteLine();
            Console.WriteLine("Orderby Year");
            var carModelQuery2 = from car in cars
                                 where car.Year > 2009
                                 orderby car.Year
                                 select car.Model;
            foreach (var c in carModelQuery2)
            {
                Console.WriteLine(c);
            }

            Console.WriteLine();
            Console.WriteLine("Orderby Model");
            carModelQuery2 = from car in cars
                                 where car.Year > 2009
                                 orderby car.Model
                                 select car.Model;
            foreach (var c in carModelQuery2)
            {
                Console.WriteLine(c);
            }
        }
    }
}
